#region Using declarations
using System;
using System.Collections;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Calculates and plots the market profile graphic
    /// </summary>
    [Description("Calculates and plots the market profile graphic")]
    public class MarketProfile : Indicator
    {
        #region Variables
        		
			private DateTime	startTime;
			private DateTime	endTime;
			private DateTime	splitTime;
		
			private TimeSpan	StartTime;
			private TimeSpan	EndTime;
			private TimeSpan	SplitTime;
		
			private double		OpenSessionPrice	= 0;
			private double		TPOPrice			= 0;
			private double		TPOLowZone			= 0;
			private double		TPOHighZone			= 0;
		
			private int			TPOValZone			= 70;
			private int			CenterIndex			= 0;
			private int			UpIndex				= 0;
			private int			DnIndex				= 0;
			private int			HighIndex			= 0;
			private int			LowIndex			= 0;
			private int			TPOHigh				= 0;
			private int 		BarCount			= 0;
			private int			TPOIndex			= 0;
			private int			TPOZone				= 0;
			private int			TPOvzHigh			= 0;
			private int			TPOvzLow			= 0;
			private int			SessionCount		= 0;
			private int			PriceInterval		= 0;
			private int 		TempBar				= 0;
		
			private int			ArrayIndex			= 0;
			private int			ArrayMid			= 2500;
			private int			TPOTotal			= 0;
			private int			ColorNumber			= 0;
			private int			ValZoneOpacity		= 2;
		
		
			private bool		Tracker 			= false;
			private bool		FirstStart			= false;
			private bool		ShowOverNight		= true;
			private bool 		CalcVar				= false;
			private bool		showsplit			= false;
			
			private Color		valzonecolor		= Color.Turquoise;
		
			private Color		Color1				= Color.Blue;
			private Color		Color2				= Color.Green;
			private Color		Color3				= Color.Magenta;
			private Color		Color4				= Color.Red;
			private Color		Color5				= Color.Turquoise;
			private Color		Color6				= Color.DarkViolet;
			private Color		Color7				= Color.DarkCyan;
			private Color		Color8				= Color.BurlyWood;
			private Color		Color9				= Color.LightCoral;
			private Color		Color10				= Color.Lime;
			private Color		Color11				= Color.Cyan;
			private Color		Color12				= Color.IndianRed;
			private Color		Color13				= Color.DarkOliveGreen;
			private Color		Color14				= Color.RosyBrown;
		
			int [] TextTrack = new int[5000]; 
			int [] TextIndex = new int[5000]; 
		
		
			private string		TextString;
			private Color		TPOColor;
		
            private string 		letters 			= ("1ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz");
		
			private DataSeries		TPOSeries;
			private DataSeries		HighValSeries;
			private DataSeries		LowValSeries;
        
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.Orange, PlotStyle.Line, "Plot0"));
			
			
			TPOSeries				= new DataSeries(this);
			HighValSeries			= new DataSeries(this);
			LowValSeries			= new DataSeries(this);
			
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            if(CurrentBar == 0)
			{
				startTime = new DateTime(Bars.SessionBegin.Year, Bars.SessionBegin.Month, Bars.SessionBegin.Day, StartTime.Hours, StartTime.Minutes, StartTime.Seconds);
				endTime = new DateTime(Bars.SessionBegin.Year, Bars.SessionBegin.Month, Bars.SessionBegin.Day, EndTime.Hours, EndTime.Minutes, EndTime.Seconds);
				splitTime = new DateTime(Bars.SessionBegin.Year, Bars.SessionBegin.Month, Bars.SessionBegin.Day, SplitTime.Hours, SplitTime.Minutes, SplitTime.Seconds);
			}
			if (CurrentBar >= 1)
			{
				if (ToTime(Time[1]) <= ToTime(startTime) && ToTime(Time[0]) > ToTime(startTime) && Time[0].DayOfWeek != DayOfWeek.Sunday)
				{
					Tracker = true;
					FirstStart = true;
					HighIndex = 0;
					LowIndex = 999999;
					TPOHigh = 0;
					BarCount = CurrentBar - 1;
					SessionCount = SessionCount + 1;
					int x = 0;
					while (x < 5000)
					{
						TextTrack[x] = 0;
						TextIndex[x] = 0;
						x++;
					}
					OpenSessionPrice = Open[0];
				}
				else if (ToTime(Time[1]) <= ToTime(endTime) && ToTime(Time[0]) > ToTime(endTime) || Time[0].DayOfWeek == DayOfWeek.Sunday && Time[1].DayOfWeek == DayOfWeek.Friday || Time[0].DayOfWeek == DayOfWeek.Monday && Time[1].DayOfWeek == DayOfWeek.Friday)
				{
					Tracker = false;
					HighIndex = 0;
					LowIndex = 999999;
					TPOHigh = 0;
					BarCount = CurrentBar - 1;
					SessionCount = SessionCount + 1;
					int x = 0;
					while (x < 5000)
					{
						TextTrack[x] = 0;
						TextIndex[x] = 0;
						x++;
					}
					OpenSessionPrice = Open[0];
				}
				
				if (ToTime(Time[0]) == ToTime(splitTime) && showsplit == true)
				{
					HighIndex = 0;
					LowIndex = 999999;
					TPOHigh = 0;
					BarCount = CurrentBar - 1;
					SessionCount = SessionCount + 1;
					int x = 0;
					while (x < 5000)
					{
						TextTrack[x] = 0;
						TextIndex[x] = 0;
						x++;
					}
					OpenSessionPrice = Open[0];
				}
								
				if (FirstStart == true && Tracker == true || ShowOverNight == true && Tracker == false && FirstStart == true) 
				{
					if (CurrentBar - BarCount <= 52)
						TextString = letters.Substring(CurrentBar - BarCount, 1);
					else
					{
						decimal Value1 = (CurrentBar - BarCount) / 52;
						TextString = letters.Substring(Convert.ToInt32(CurrentBar - BarCount - Math.Floor(Value1) * 52), 1);
					}
					
					if (CurrentBar - BarCount <= 14)
						ColorNumber = CurrentBar - BarCount;
					else 
					{
						decimal Value1 = (CurrentBar - BarCount) / 14;
						ColorNumber = Convert.ToInt32(CurrentBar - BarCount - Math.Floor(Value1) * 14 - 1);
					}				
					if(ColorNumber == 1)
						TPOColor = Color1;
					else if (ColorNumber == 2)
						TPOColor = Color2;
					else if (ColorNumber == 3)
						TPOColor = Color3;	
					else if (ColorNumber == 4)
						TPOColor = Color4;
					else if (ColorNumber == 5)
						TPOColor = Color5;
					else if (ColorNumber == 6)
						TPOColor = Color6;
					else if (ColorNumber == 7)
						TPOColor = Color7;	
					else if (ColorNumber == 8)
						TPOColor = Color8;
					else if (ColorNumber == 9)
						TPOColor = Color9;
					else if (ColorNumber == 10)
						TPOColor = Color10;
					else if (ColorNumber == 11)
						TPOColor = Color11;
					else if (ColorNumber == 12)
						TPOColor = Color12;	
					else if (ColorNumber == 13)
						TPOColor = Color13;
					else if (ColorNumber == 14)
						TPOColor = Color14;
					
					UpIndex = Convert.ToInt32(Math.Round((High[0] - Open[0]) / TickSize, 0, MidpointRounding.AwayFromZero));
					DnIndex = Convert.ToInt32(Math.Round((Open[0] - Low[0]) / TickSize, 0, MidpointRounding.AwayFromZero));
					
					int y = 0;
					while (y <= UpIndex)
					{
						ArrayIndex = Convert.ToInt32(Math.Round((ArrayMid + ((Open[0] + y * TickSize) - OpenSessionPrice) / TickSize), 0, MidpointRounding.AwayFromZero));
						if (ArrayIndex > HighIndex)
							HighIndex = ArrayIndex;
						if (ArrayIndex < LowIndex)
							LowIndex = ArrayIndex;
						if (CurrentBar - BarCount > TextIndex[ArrayIndex])
						{
							DrawText("Text" + ArrayIndex + CurrentBar, TextString, CurrentBar - BarCount - TextTrack[ArrayIndex], Open[0] + y * TickSize, TPOColor);
							TextTrack[ArrayIndex] = TextTrack[ArrayIndex] + 1;
							if (TextTrack[ArrayIndex] > TPOHigh)
								TPOHigh = TextTrack[ArrayIndex];
							TextIndex[ArrayIndex] = CurrentBar - BarCount;
							CalcVar = true;
							
						}
						//DrawText("Text" + ArrayIndex + CurrentBar, TextString, 0, Open[0] + y * TickSize, TPOColor);
						y++;
						
					}
					
					y = 1;
					
					while (y <= DnIndex)
						{
						ArrayIndex = Convert.ToInt32(Math.Round((ArrayMid + ((Open[0] - y * TickSize) - OpenSessionPrice) / TickSize), 0, MidpointRounding.AwayFromZero));
						if (ArrayIndex > HighIndex)
							HighIndex = ArrayIndex;
						if (ArrayIndex < LowIndex)
							LowIndex = ArrayIndex;
						if (CurrentBar - BarCount > TextIndex[ArrayIndex] || CurrentBar - BarCount == 0)
						{
							DrawText("Text" + ArrayIndex + CurrentBar, TextString, CurrentBar - BarCount - TextTrack[ArrayIndex], Open[0] - y * TickSize, TPOColor);
							TextTrack[ArrayIndex] = TextTrack[ArrayIndex] + 1;
							if (TextTrack[ArrayIndex] > TPOHigh)
								TPOHigh = TextTrack[ArrayIndex];
							TextIndex[ArrayIndex] = CurrentBar - BarCount;
							CalcVar = true;
						}
						//DrawText("Text" + ArrayIndex + CurrentBar, TextString, 0, Open[0] - y * TickSize, TPOColor);
						y++;
					}
					
					if (CalcVar == true && HighIndex - LowIndex != 0)	
					{	
					CalcVar = false;
					TPOTotal = 0;
					int x = 0;
					while (x <= HighIndex)
					{
						TPOTotal = TPOTotal + TextTrack[x];
						x++;
					}
					
					CenterIndex = Convert.ToInt32(Math.Round((HighIndex - LowIndex) * 0.5, 0, MidpointRounding.AwayFromZero));
					
					x = 0;
										
					while (x <= CenterIndex && CenterIndex != 0)
					{
						if (TextTrack[CenterIndex + LowIndex + x] == TPOHigh)
						{
							TPOIndex = CenterIndex + LowIndex + x;
							break;
						}
						else if (TextTrack[CenterIndex + LowIndex - x] == TPOHigh)
						{
							TPOIndex = CenterIndex + LowIndex - x;
							break;
						}
						else
							x++;	
					}
					
					TPOPrice = OpenSessionPrice + (TPOIndex - ArrayMid) * TickSize;
					TPOZone = Convert.ToInt32(Math.Round(TPOTotal * TPOValZone * .01, 0, MidpointRounding.AwayFromZero));
					
					x= TextTrack[TPOIndex];
					y = 1;
					int z = 1;
					int upTPO;
					int dnTPO;
					
					while (x <= TPOZone)
						
					{
						upTPO = TextTrack[TPOIndex + y] + TextTrack[TPOIndex + y + 1];
						dnTPO = TextTrack[TPOIndex - z] + TextTrack[TPOIndex - z - 1];
						if (upTPO == dnTPO)
						{
							x = x + upTPO + dnTPO;
							TPOvzHigh = TPOIndex + y + 1;
							TPOvzLow = TPOIndex - z - 1;
							y = y + 2;
							z = z + 2;
						}
						else if (upTPO > dnTPO)
						{
							x = x + upTPO;
							TPOvzHigh = TPOIndex + y + 1;
							y = y + 2;
							if (z == 1)
								TPOvzLow = TPOIndex;
						}
						else if (upTPO < dnTPO)
						{
							x = x + dnTPO;
							TPOvzLow = TPOIndex - z - 1;
							z = z + 2;
							if (y == 1)
								TPOvzHigh = TPOIndex;
						}
						
					}
					
					TPOHighZone = OpenSessionPrice + (TPOvzHigh - ArrayMid) * TickSize;
					TPOLowZone = OpenSessionPrice + (TPOvzLow - ArrayMid) * TickSize;
					
					TPOSeries.Set(TPOPrice);
					HighValSeries.Set(TPOHighZone);
					LowValSeries.Set(TPOLowZone);
										
					if (CurrentBar - BarCount >= 1)
					{
						DrawLine("TPO" + SessionCount, false, CurrentBar - BarCount, TPOPrice, CurrentBar - BarCount - TPOHigh, TPOPrice, Color.Magenta, DashStyle.Solid, 1);
						DrawLine("TPOvzHigh" + SessionCount, false, CurrentBar - BarCount, TPOHighZone, CurrentBar - BarCount - TPOHigh, TPOHighZone, Color.DodgerBlue, DashStyle.Solid, 1);
						DrawLine("TPOvzLow" + SessionCount, false, CurrentBar - BarCount, TPOLowZone, CurrentBar - BarCount - TPOHigh, TPOLowZone, Color.DodgerBlue, DashStyle.Solid, 1);
						DrawRectangle("ValZone" + SessionCount, false, CurrentBar - BarCount, TPOLowZone, CurrentBar - BarCount - TPOHigh, TPOHighZone, Color.Transparent, valzonecolor, ValZoneOpacity);

					}
					}
				}
			}
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Plot0
        {
            get { return Values[0]; }
        }
		
		[Description("Value Zone Percentage")]
        [Category("Parameters")]
        public int TPOvalzone
        {
            get { return TPOValZone; }
            set { TPOValZone = Math.Max(1, value); }
        }
		
		[Description("Show overnight session")]
        [Category("Parameters")]
        public bool showovernight
        {
            get { return ShowOverNight; }
            set { ShowOverNight = value; }
        }
		
		[Description("Split Day Session")]
        [Category("Parameters")]
        public bool ShowSplit
        {
            get { return showsplit; }
            set { showsplit = value; }
        }
		
       [Description("Market Profile Start Time")]
		[Category("Settings")]
		[Gui.Design.DisplayName("\t\t\t\t\t\tStart Time")]
		public TimeSpan MPStartTime
		{
			get { return StartTime; }
			set { StartTime = value; }
		}
		
		[Description("Market Profile End Time")]
		[Category("Settings")]
		[Gui.Design.DisplayName("\t\t\t\t\tEnd Time")]
		public TimeSpan MPEndTime
		{
			get { return EndTime; }
			set { EndTime = value; }
		} 
		
		[Description("Split Time")]
		[Category("Settings")]
		[Gui.Design.DisplayName("\t\t\t\t\tSplit Time")]
		public TimeSpan MPSplitTime
		{
			get { return SplitTime; }
			set { SplitTime = value; }
		}
		
		[XmlIgnore()]
		[Description("Value Zone Color Shading")]
		[Category("Visual")]
		[Gui.Design.DisplayNameAttribute("Value Zone Color")]
		public Color ValZoneColor
		{
			get { return valzonecolor; }
			set { valzonecolor = value; }
		}
		
		[XmlIgnore()]
		[Description("Value Zone Color opacity.")]
		[Category("Visual")]
		[Gui.Design.DisplayNameAttribute("Opacity")]
		public int valzoneopacity
		{
			get { return ValZoneOpacity; }
			set { ValZoneOpacity = Math.Max(0, value); }
		}
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private MarketProfile[] cacheMarketProfile = null;

        private static MarketProfile checkMarketProfile = new MarketProfile();

        /// <summary>
        /// Calculates and plots the market profile graphic
        /// </summary>
        /// <returns></returns>
        public MarketProfile MarketProfile(bool showovernight, bool showSplit, int tPOvalzone)
        {
            return MarketProfile(Input, showovernight, showSplit, tPOvalzone);
        }

        /// <summary>
        /// Calculates and plots the market profile graphic
        /// </summary>
        /// <returns></returns>
        public MarketProfile MarketProfile(Data.IDataSeries input, bool showovernight, bool showSplit, int tPOvalzone)
        {
            checkMarketProfile.showovernight = showovernight;
            showovernight = checkMarketProfile.showovernight;
            checkMarketProfile.ShowSplit = showSplit;
            showSplit = checkMarketProfile.ShowSplit;
            checkMarketProfile.TPOvalzone = tPOvalzone;
            tPOvalzone = checkMarketProfile.TPOvalzone;

            if (cacheMarketProfile != null)
                for (int idx = 0; idx < cacheMarketProfile.Length; idx++)
                    if (cacheMarketProfile[idx].showovernight == showovernight && cacheMarketProfile[idx].ShowSplit == showSplit && cacheMarketProfile[idx].TPOvalzone == tPOvalzone && cacheMarketProfile[idx].EqualsInput(input))
                        return cacheMarketProfile[idx];

            MarketProfile indicator = new MarketProfile();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.showovernight = showovernight;
            indicator.ShowSplit = showSplit;
            indicator.TPOvalzone = tPOvalzone;
            indicator.SetUp();

            MarketProfile[] tmp = new MarketProfile[cacheMarketProfile == null ? 1 : cacheMarketProfile.Length + 1];
            if (cacheMarketProfile != null)
                cacheMarketProfile.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheMarketProfile = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Calculates and plots the market profile graphic
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MarketProfile MarketProfile(bool showovernight, bool showSplit, int tPOvalzone)
        {
            return _indicator.MarketProfile(Input, showovernight, showSplit, tPOvalzone);
        }

        /// <summary>
        /// Calculates and plots the market profile graphic
        /// </summary>
        /// <returns></returns>
        public Indicator.MarketProfile MarketProfile(Data.IDataSeries input, bool showovernight, bool showSplit, int tPOvalzone)
        {
            return _indicator.MarketProfile(input, showovernight, showSplit, tPOvalzone);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Calculates and plots the market profile graphic
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MarketProfile MarketProfile(bool showovernight, bool showSplit, int tPOvalzone)
        {
            return _indicator.MarketProfile(Input, showovernight, showSplit, tPOvalzone);
        }

        /// <summary>
        /// Calculates and plots the market profile graphic
        /// </summary>
        /// <returns></returns>
        public Indicator.MarketProfile MarketProfile(Data.IDataSeries input, bool showovernight, bool showSplit, int tPOvalzone)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.MarketProfile(input, showovernight, showSplit, tPOvalzone);
        }

    }
}
#endregion
